package org.warp.commonutils.concurrency.executor;

import java.time.Duration;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.function.BiConsumer;
import org.jetbrains.annotations.Nullable;

public class BoundedExecutorService {

	private static final int MAX_BLOCKING_QUEUE_SIZE = 50000;

	private BoundedExecutorService() {

	}

	@Deprecated
	public static BlockingOnFullQueueExecutorServiceDecorator createUnbounded(
			int maxPoolSize,
			long keepAliveTime,
			TimeUnit unit,
			@Nullable BiConsumer<Boolean, Integer> queueSizeStatus) {
		return create(0, maxPoolSize, keepAliveTime, unit, Executors.defaultThreadFactory(), queueSizeStatus);
	}

	public static BlockingOnFullQueueExecutorServiceDecorator createUnbounded(
			int maxPoolSize,
			long keepAliveTime,
			TimeUnit unit,
			ThreadFactory threadFactory,
			@Nullable BiConsumer<Boolean, Integer> queueSizeStatus) {
		return createCustom(0, maxPoolSize, keepAliveTime, unit, threadFactory, Duration.ofDays(100000), queueSizeStatus, new LinkedBlockingQueue<>());
	}

	public static BlockingOnFullQueueExecutorServiceDecorator createUnbounded(
			int maxPoolSize,
			long keepAliveTime,
			TimeUnit unit,
			ThreadFactory threadFactory,
			@Nullable BiConsumer<Boolean, Integer> queueSizeStatus,
			BlockingQueue<Runnable> queue) {
		return createCustom(0, maxPoolSize, keepAliveTime, unit, threadFactory, Duration.ofDays(100000), queueSizeStatus, queue);
	}

	@Deprecated
	public static BlockingOnFullQueueExecutorServiceDecorator create(
			int maxQueueSize,
			int maxPoolSize,
			long keepAliveTime,
			TimeUnit unit,
			@Nullable BiConsumer<Boolean, Integer> queueSizeStatus) {
		return create(maxQueueSize, maxPoolSize, keepAliveTime, unit, Executors.defaultThreadFactory(), queueSizeStatus);
	}

	public static BlockingOnFullQueueExecutorServiceDecorator create(
			int maxQueueSize,
			int maxPoolSize,
			long keepAliveTime,
			TimeUnit unit,
			ThreadFactory threadFactory,
			@Nullable BiConsumer<Boolean, Integer> queueSizeStatus) {
		return createCustom(maxQueueSize, maxPoolSize, keepAliveTime, unit, threadFactory, Duration.ofDays(100000), queueSizeStatus, new LinkedBlockingQueue<>(maxQueueSize));
	}

	public static BlockingOnFullQueueExecutorServiceDecorator createCustom(
			int maxQueueSize,
			int maxPoolSize,
			long keepAliveTime,
			TimeUnit unit,
			ThreadFactory threadFactory,
			Duration queueItemTtl,
			@Nullable BiConsumer<Boolean, Integer> queueSizeStatus,
			BlockingQueue<Runnable> queue) {
		ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(maxPoolSize,
				maxPoolSize,
				keepAliveTime,
				unit,
				queue,
				threadFactory
		);
		if (keepAliveTime > 0) {
			threadPoolExecutor.allowCoreThreadTimeOut(true);
		}
		threadPoolExecutor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
		return new BlockingOnFullQueueExecutorServiceDecorator(threadPoolExecutor,
				maxQueueSize,
				queueItemTtl,
				queue::size,
				queueSizeStatus
		);
	}
}
