package org.warp.commonutils.type;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Spliterator;
import java.util.function.Consumer;
import java.util.function.IntFunction;
import java.util.function.Predicate;
import java.util.stream.Stream;
import org.jetbrains.annotations.NotNull;

public class ImmutableLinkedSet<E> implements Set<E> {

	private final LinkedHashSet<E> set;

	public ImmutableLinkedSet(LinkedHashSet<E> set) {
		this.set = set;
	}

	@Override
	public int size() {
		return set.size();
	}

	@Override
	public boolean isEmpty() {
		return set.isEmpty();
	}

	@Override
	public boolean contains(Object o) {
		return set.contains(o);
	}

	@NotNull
	@Override
	public Iterator<E> iterator() {
		var it = set.iterator();
		return new Iterator<E>() {
			@Override
			public boolean hasNext() {
				return it.hasNext();
			}

			@Override
			public E next() {
				return it.next();
			}
		};
	}

	@Override
	public void forEach(Consumer<? super E> action) {
		set.forEach(action);
	}

	@NotNull
	@Override
	public Object[] toArray() {
		return set.toArray();
	}

	@NotNull
	@Override
	public <T> T[] toArray(@NotNull T[] a) {
		//noinspection SuspiciousToArrayCall
		return set.toArray(a);
	}

	@Override
	public <T> T[] toArray(IntFunction<T[]> generator) {
		//noinspection SuspiciousToArrayCall
		return set.toArray(generator);
	}

	@Override
	public boolean add(E e) {
		throw new UnsupportedOperationException("Read only");
	}

	@Override
	public boolean remove(Object o) {
		throw new UnsupportedOperationException("Read only");
	}

	@Override
	public boolean containsAll(@NotNull Collection<?> c) {
		return set.containsAll(c);
	}

	@Override
	public boolean addAll(@NotNull Collection<? extends E> c) {
		throw new UnsupportedOperationException("Read only");
	}

	@Override
	public boolean retainAll(@NotNull Collection<?> c) {
		throw new UnsupportedOperationException("Read only");
	}

	@Override
	public boolean removeAll(@NotNull Collection<?> c) {
		throw new UnsupportedOperationException("Read only");
	}

	@Override
	public boolean removeIf(Predicate<? super E> filter) {
		throw new UnsupportedOperationException("Read only");
	}

	@Override
	public void clear() {
		throw new UnsupportedOperationException("Read only");
	}

	@Override
	public Spliterator<E> spliterator() {
		var spl = set.spliterator();
		return new Spliterator<E>() {
			@Override
			public boolean tryAdvance(Consumer<? super E> action) {
				return spl.tryAdvance(action);
			}

			@Override
			public Spliterator<E> trySplit() {
				return spl.trySplit();
			}

			@Override
			public long estimateSize() {
				return spl.estimateSize();
			}

			@Override
			public int characteristics() {
				return spl.characteristics();
			}
		};
	}

	@Override
	public Stream<E> stream() {
		return set.stream();
	}

	@Override
	public Stream<E> parallelStream() {
		return set.parallelStream();
	}

	public LinkedHashSet<E> copyMutable() {
		return new LinkedHashSet<>(set);
	}
}
